"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
require("rxjs/add/operator/takeUntil");
const fs_1 = require("fs");
const config_interface_1 = require("./config.interface");
require("rxjs/add/observable/forkJoin");
const Handlebars = require('handlebars');
const path = require("path");
class Generator {
    constructor(config, template) {
        this.config = config;
        this.template = template;
        this.gitlabApi = require('gitlab-kirakishin')(config.gitlab);
        this.logger.info('generator initialized');
    }
    generate() {
        this.logger.log(Generator.debugPrefix, 'generate prepared');
        let changelogData = {
            title: 'Changelog',
            releases: []
        };
        return rxjs_1.Observable.create((generationStatus) => {
            this.logger.info(Generator.debugPrefix, 'generating changelog...');
            this.awaitMilestones()
                .subscribe((milestones) => {
                let milestonesDataProcessings = [];
                milestones = milestones.sort((a, b) => b.id - a.id);
                for (let ms of milestones) {
                    this.processMilestoneData(ms, milestonesDataProcessings, changelogData);
                }
                this.onAllMilestonesDataProcessed(milestonesDataProcessings)
                    .subscribe(() => {
                    if (this.processChangelogDataToMarkdownFile(changelogData)) {
                        this.logger.info(Generator.debugPrefix, 'changelog generated');
                        generationStatus.next('OK');
                    }
                    else {
                        this.logger.info(Generator.debugPrefix, 'changelog generation error');
                        generationStatus.next('changelog generation error');
                    }
                });
            });
        });
    }
    get logger() {
        let e = () => {
        };
        if (this.config.logLevel >= config_interface_1.LOG_LEVEL.OFF) {
            return {
                error: console.error.bind(console.error, Generator.debugPrefix),
                warn: this.config.logLevel >= config_interface_1.LOG_LEVEL.WARN ? console.warn.bind(console.warn, Generator.debugPrefix) : e,
                info: this.config.logLevel >= config_interface_1.LOG_LEVEL.INFO ? console.info.bind(console.info, Generator.debugPrefix) : e,
                log: this.config.logLevel >= config_interface_1.LOG_LEVEL.LOG ? console.log.bind(console.log, Generator.debugPrefix) : e
            };
        }
        return { error: console.error.bind(console.error, Generator.debugPrefix), warn: e, info: e, log: e };
    }
    processChangelogDataToMarkdownFile(changelogData) {
        let file = this.config.destFile;
        this.logger.log(`saving content in file: ${file}`);
        let generator = Handlebars.compile(this.template);
        let content = generator(changelogData);
        try {
            fs_1.writeFileSync(path.resolve(file), content, 'utf-8');
            this.logger.info(`changelog saved in file: ${file}`);
        }
        catch (e) {
            this.logger.error(`fail to save in file: ${file}`, e);
            return false;
        }
        return true;
    }
    onAllMilestonesDataProcessed(milestonesDataProcessings) {
        return rxjs_1.Observable.create((observer) => {
            this.logger.log('waiting for all milestones MRs receives');
            rxjs_1.Observable.forkJoin(...milestonesDataProcessings)
                .subscribe((allMilestoneMergeRequestsStatus) => {
                if (allMilestoneMergeRequestsStatus.every((elt) => elt !== 0)) {
                    this.logger.log('all milestones MRs received');
                    observer.next(true);
                }
            });
        });
    }
    processMilestoneData(ms, allRequestsObservables, changelogData) {
        let title = `${(ms.title || 'unknown')}`;
        this.logger.log(`process milestone data for #${ms.id} - ${title}`);
        let milestoneData = {
            title: title,
            changes: []
        };
        changelogData.releases.push(milestoneData);
        let fetchingRequest = new rxjs_1.ReplaySubject(0);
        this.processMilestoneMergeRequestsData(ms.id, milestoneData, fetchingRequest);
        allRequestsObservables.push(fetchingRequest);
    }
    processMilestoneMergeRequestsData(id, milestoneData, fetchingRequest) {
        this.awaitMilestoneMergeRequests(id)
            .subscribe((milestoneMergeRequests) => {
            this.logger.log(`got MRs of milestone #${id}`);
            if (milestoneMergeRequests && milestoneMergeRequests.length > 0) {
                for (let mr of milestoneMergeRequests) {
                    let change = {
                        id: mr.id,
                        title: mr.title || `Merge Request #${mr.id}`,
                        link: mr.web_url || ''
                    };
                    this.logger.log(`got MR #${mr.id} of milestone #${id}:`, change.title);
                    milestoneData.changes.push(change);
                }
            }
            this.logger.log(`send that the MRs of milestone #${id} are received with success`);
            fetchingRequest.next(1);
            fetchingRequest.complete();
        }, (error) => {
            this.logger.log(`send that the MRs of milestone #${id} are in error`);
            fetchingRequest.next(-1);
            fetchingRequest.complete();
        });
    }
    awaitMilestones() {
        return this.toObservable(this.gitlabApi.projects.milestones.all.bind(null, this.config.projectId));
    }
    awaitMilestoneMergeRequests(milestoneId) {
        return this.toObservable(this.gitlabApi.projects.milestones.allMergeRequests.bind(null, this.config.projectId, milestoneId));
    }
    toObservable(cb) {
        return rxjs_1.Observable.create((observer) => {
            if (cb && typeof cb === 'function') {
                let res;
                try {
                    res = cb((data) => {
                        observer.next(data);
                        observer.complete();
                    });
                }
                catch (e) {
                    observer.error('cb error');
                }
            }
        });
    }
}
Generator.debugPrefix = '[gitlab-changelog-generator]';
exports.Generator = Generator;
